// This silly little Fiddler extension flips all GIF/PNG/JPG images upside-down as they flow across the wire.
// It introduces a "Flip Images" item on the Tools menu that enables or disables the extension.
using System;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using Fiddler;
using System.Windows.Forms;

[assembly: Fiddler.RequiredVersion("2.4.6.0")]
namespace AustralianImages
{
    public class ImageFlipper: IAutoTamper
    {
        private bool bFlipEnabled = false;
        private bool bBlurEnabled = false;
        MenuItem oFlipMenuItem = new MenuItem("F&lip Images");
        MenuItem oBlurMenuItem = new MenuItem("Blur Images");

        #region IAutoTamper Members
        public void OnBeforeReturningError(Session oSession) { /* no-op */}
        public void AutoTamperRequestAfter(Session oSession){/* no-op */}
        public void AutoTamperRequestBefore(Session oSession){/* no-op */}
        public void AutoTamperResponseAfter(Session oSession){/* no-op */}
        public void AutoTamperResponseBefore(Session oSession)
        {
            if (!bFlipEnabled && !bBlurEnabled) return;

            if (null != oSession && oSession.responseCode == 200 && null != oSession.oResponse && null != oSession.oResponse.headers 
                && (oSession.oResponse.headers.ExistsAndContains("Content-Type", "image/gif") ||
                oSession.oResponse.headers.ExistsAndContains("Content-Type", "image/png") ||
                 oSession.oResponse.headers.ExistsAndContains("Content-Type", "image/jp"))
                ){
                try
                    {
                        oSession.utilDecodeResponse();
                        using (MemoryStream oStream = new MemoryStream(oSession.responseBodyBytes))
                        {
                            Bitmap oBMP = new Bitmap(oStream);

                            if (bFlipEnabled)
                            {
                                oBMP.RotateFlip(RotateFlipType.Rotate180FlipNone);
                            }

                            using (MemoryStream oNewStream = new MemoryStream())
                            {
                                if (bBlurEnabled)
                                {
                                    ImageCodecInfo jgpEncoder = GetEncoder(ImageFormat.Jpeg);

                                    // Create an Encoder object based on the GUID for the Quality parameter category.
                                    Encoder myEncoder = Encoder.Quality;
                                    EncoderParameters myEncoderParameters = new EncoderParameters(1);

                                    EncoderParameter myEncoderParameter = new EncoderParameter(myEncoder, 8L);
                                    myEncoderParameters.Param[0] = myEncoderParameter;

                                    oBMP.Save(oNewStream, jgpEncoder, myEncoderParameters);
                                }
                                else
                                {
                                    oBMP.Save(oNewStream, ImageFormat.Png);
                                }
                                oSession.responseBodyBytes = oNewStream.ToArray();
                                oSession.oResponse.headers["Content-Type"] = (bBlurEnabled) ? "image/jpeg" : "image/png";

                                oSession.oResponse.headers["Cache-Control"] = "no-cache";
                                oSession.oResponse.headers["Content-Length"] = oSession.responseBodyBytes.Length.ToString();
                            }
                        }
                    }
                    catch (Exception eX)
                    {
                        System.Diagnostics.Trace.WriteLine(eX.ToString(), "ImageFlipper Failed");
                    }
            }
        }
        #endregion



        private ImageCodecInfo GetEncoder(ImageFormat format)
        {

            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageDecoders();

            foreach (ImageCodecInfo codec in codecs)
            {
                if (codec.FormatID == format.Guid)
                {
                    return codec;
                }
            }
            return null;
        }


        #region IFiddlerExtension Members

        public void OnBeforeUnload()
        {
            bBlurEnabled = bFlipEnabled = false;
            oFlipMenuItem.Dispose();
            oBlurMenuItem.Dispose();
        }

        public void OnLoad()
        {
            oFlipMenuItem.Click += delegate(object s, EventArgs e)
            {
                oFlipMenuItem.Checked = !oFlipMenuItem.Checked;
                bFlipEnabled = oFlipMenuItem.Checked;
            };
            oBlurMenuItem.Click += delegate(object s, EventArgs e)
            {
                oBlurMenuItem.Checked = !oBlurMenuItem.Checked;
                bBlurEnabled = oBlurMenuItem.Checked;
            };
            FiddlerApplication.UI.mnuTools.MenuItems.AddRange(new[] { oFlipMenuItem, oBlurMenuItem });
        }

        #endregion
    }
}
